#include "Cube.h"

Cube::Cube (double s)
{
	size = s;
}

Cube::~Cube ()
{
	// nothing to do
}

void Cube::Draw ()
{
	glColor4fv (color);
	glutSolidCube (size);
}

double Cube::GetBoundingRadius ()
{
	// corners have the distance of size * sqrt (3) / two.
	return size * 1.732051 / 2;
}

BoundingBox Cube::GetBoundingBox ()
{
	BoundingBox result;

	result.xmin = -size / 2;
	result.xmax = size / 2;
	result.ymin = -size / 2;
	result.ymax = size / 2;
	result.zmin = -size / 2;
	result.zmax = size / 2;

	return result;
}

Cube2::Cube2 (float s)
{
	size = s;
	
	Construct (s);
	ComputeNormals (true);

	PrintToConsole ();
}

Cube2::~Cube2 ()
{
	if (v_data != NULL) delete v_data;
	if (indices != NULL) delete indices;
}

void Cube2::Construct (float size)
{
	// cube has 8 vertices, construct them
	v_data = new float[3*8];

	v_data [0] = size/2;
	v_data [1] = size/2;
	v_data [2] = size/2;
 
	v_data [3] = size/2;
	v_data [4] = -size/2;
	v_data [5] = size/2;

	v_data [6] = -size/2;
	v_data [7] = -size/2;
	v_data [8] = size/2;

	v_data [9] = -size/2;
	v_data [10] = size/2;
	v_data [11] = size/2;
 
	v_data [12] = size/2;
	v_data [13] = size/2;
	v_data [14] = -size/2;

	v_data [15] = size/2;
	v_data [16] = -size/2;
	v_data [17] = -size/2;
 
	v_data [18] = -size/2;
	v_data [19] = -size/2;
	v_data [20] = -size/2;

	v_data [21] = -size/2;
	v_data [22] = size/2;
	v_data [23] = -size/2;

	// cube consists of 6 sides composed of 2 triangles each, construct them
	indices = new GLuint[12 * 3];

	indices [0] = 0;
	indices [1] = 4;
	indices [2] = 5;

	indices [3] = 0;
	indices [4] = 5;
	indices [5] = 1;

	indices [6] = 1;
	indices [7] = 5;
	indices [8] = 6;

	indices [9] = 1;
	indices [10] = 6;
	indices [11] = 2;

	indices [12] = 2;
	indices [13] = 6;
	indices [14] = 7;

	indices [15] = 2;
	indices [16] = 7;
	indices [17] = 3;

	indices [18] = 3;
	indices [19] = 7;
	indices [20] = 4;

	indices [21] = 3;
	indices [22] = 4;
	indices [23] = 0;

	indices [24] = 0;
	indices [25] = 1;
	indices [26] = 2;

	indices [27] = 0;
	indices [28] = 2;
	indices [29] = 3;

	indices [30] = 4;
	indices [31] = 6;
	indices [32] = 5;

	indices [33] = 4;
	indices [34] = 7;
	indices [35] = 6;

	SetVertexData (v_data, 8);
	SetVertexIndices (indices, 12);

}

TexturedCube::TexturedCube (float size_, GLuint texture_id_, bool allow_lighting_)
{
	half_size = size_/2;
	texture_id = texture_id_;
	allow_lighting = allow_lighting_;

	ConstructDisplayList ();
}

TexturedCube::~TexturedCube ()
{
	glDeleteLists (list_id, 1);
}

double TexturedCube::GetBoundingRadius ()
{
	return half_size * 1.732051;
}

BoundingBox TexturedCube::GetBoundingBox ()
{
	BoundingBox result;

	result.xmin = -half_size;
	result.xmax = half_size;
	result.ymin = -half_size;
	result.ymax = half_size;
	result.zmin = -half_size;
	result.zmax = half_size;

	return result;
}

void TexturedCube::ConstructDisplayList ()
{
	GLboolean lighting_state;
	GLboolean texture_state;

	list_id =  glGenLists (1);

	glNewList (list_id, GL_COMPILE);

	// get states (note: states are taken at compile time of the display list !!!)
	glGetBooleanv (GL_TEXTURE_2D, &texture_state);
	glEnable(GL_TEXTURE_2D);	
	
	if (!allow_lighting)
	{
		glGetBooleanv (GL_LIGHTING, &lighting_state);
		glDisable (GL_LIGHTING);
	}

	glBindTexture (GL_TEXTURE_2D, texture_id);
	glColor4fv (color);

	glBegin (GL_QUADS);
		
		// top
		glNormal3f (0.0f, 0.0f, 1.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (half_size, half_size, half_size);
		glNormal3f (0.0f, 0.0f, 1.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (half_size, -half_size, half_size);
		glNormal3f (0.0f, 0.0f, 1.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (-half_size, -half_size, half_size);
		glNormal3f (0.0f, 0.0f, 1.0f);glTexCoord2f (0.0f, 1.0f); glVertex3f (-half_size, half_size, half_size);

		// bottom
		glNormal3f (0.0f, 0.0f, -1.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (half_size, half_size, -half_size);
		glNormal3f (0.0f, 0.0f, -1.0f);glTexCoord2f (0.0f, 1.0f); glVertex3f (-half_size, half_size, -half_size);
		glNormal3f (0.0f, 0.0f, -1.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (-half_size, -half_size, -half_size);
		glNormal3f (0.0f, 0.0f, -1.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (half_size, -half_size, -half_size);

		// front
		glNormal3f (1.0f, 0.0f, 0.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (half_size, half_size, -half_size);
		glNormal3f (1.0f, 0.0f, 0.0f); glTexCoord2f (0.0f, 1.0f); glVertex3f (half_size, -half_size, -half_size);
		glNormal3f (1.0f, 0.0f, 0.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (half_size, -half_size, half_size);
		glNormal3f (1.0f, 0.0f, 0.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (half_size, half_size, half_size);

		// back
		glNormal3f (-1.0f, 0.0f, 0.0f); glTexCoord2f (0.0f, 1.0f); glVertex3f (-half_size, half_size, -half_size);
		glNormal3f (-1.0f, 0.0f, 0.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (-half_size, half_size, half_size);
		glNormal3f (-1.0f, 0.0f, 0.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (-half_size, -half_size, half_size);
		glNormal3f (-1.0f, 0.0f, 0.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (-half_size, -half_size, -half_size);

		// right
		glNormal3f (0.0f, 1.0f, 0.0f); glTexCoord2f (0.0f, 1.0f); glVertex3f (-half_size, half_size, -half_size);
		glNormal3f (0.0f, 1.0f, 0.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (half_size, half_size, -half_size);
		glNormal3f (0.0f, 1.0f, 0.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (half_size, half_size, half_size);
		glNormal3f (0.0f, 1.0f, 0.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (-half_size, half_size, half_size);

		// left
		glNormal3f (0.0f, -1.0f, 0.0f); glTexCoord2f (0.0f, 0.0f); glVertex3f (-half_size, -half_size, -half_size);
		glNormal3f (0.0f, -1.0f, 0.0f); glTexCoord2f (1.0f, 0.0f); glVertex3f (-half_size, -half_size, half_size);
		glNormal3f (0.0f, -1.0f, 0.0f); glTexCoord2f (1.0f, 1.0f); glVertex3f (half_size, -half_size, half_size);
		glNormal3f (0.0f, -1.0f, 0.0f); glTexCoord2f (0.0f, 1.0f); glVertex3f (half_size, -half_size, -half_size);

	glEnd ();

	// restore states
	if (!allow_lighting && lighting_state) glEnable (GL_LIGHTING);
	if (!texture_state) glDisable (GL_TEXTURE_2D);

	glEndList ();
}

void TexturedCube::Draw ()
{
	glCallList (list_id);
}
