#ifndef LANDSCAPE_SCENE_
#define LANDSCAPE_SCENE_

#include "Scene.h"
#include "Geometry.h"
#include "GroupObject.h"

/**
 * \brief A landscape generated by perlin noise.
 *
 * Scene with a landscape generated by perlin noise. The landscape consists of a set of patches with size
 * 64x64. These patches are constructed from a heightfield that consists of 513x513 values. Additionally,
 * several instances of the same height field are repeated along the x and y direction to make
 * the are even bigger (those components are rotated randomly in order to make this repetition less
 * appearant to the eye).
 */
class LandscapeScene: public Scene
{
public:
	/**
	 * Builds the scene.
	 */
	void Init ();

private:
	/**  The height field of the landscape */
	float * height_field;

	/*
	 * Generates a part of the landscape.
	 */
	GraphicalObject * GeneratePart (float * height_field, int min_x, int max_x, int min_y, int max_y, int total_x, int total_y, float stride);

	/*
	 * Replicates one patch of the landscape
	 */
	GroupObject * ReplicatePatches (GraphicalObject * one_patch, float size, int min_x, int max_x, int min_y, int max_y);

	/*
	 * Utility routine.
	 */
	void RandomRotate (GraphicalObject * patch, float x_offset, float y_offset, float size);

};

#endif