#include "TorusScene.h"
#include "SimpleArrow.h"
#include "Torus.h"
#include "Cube.h"
#include "Icosahedron.h"
#include "Teapot.h"
#include "TerminalObject.h"
#include "DLODObject.h"
#include "Textures.h"

#include "TorusGenerator.h"

void TorusScene::Init ()
{
	Torus2 * torus1, * torus2, * torus3, * torus4, * torus5, *torus6;
	GraphicalObject * terminal_object;
	DLODObject * dlod_object;

	RawObject * space;

	RawObject * torus;
	RawObject * arrow;
	RawObject * cube;
	RawObject * icosahedron;
	RawObject * teapot;

	Light * light;

	GraphicalObject * big_torus;


	// create the different levels of detail
	torus1 = new Torus2 (2.0f, 1.0f, 6, 6);
	torus1->SetTexture (Textures::GetId (T_PERLIN3));
	torus1->SetColor (1.0f, 1.0f, 1.0f, 1.0f);
	torus2 = new Torus2 (2.0f, 1.0f, 10, 10);
	torus2->SetTexture (Textures::GetId (T_PERLIN3));
	torus2->SetColor (1.0f, 1.0f, 1.0f, 1.0f);
	torus3 = new Torus2 (2.0f, 1.0f, 16, 16);
	torus3->SetTexture (Textures::GetId (T_PERLIN3));
	torus3->SetColor (1.0f, 1.0f, 1.0f, 1.0f);
	torus4 = new Torus2 (2.0f, 1.0f, 32, 32);
	torus4->SetTexture (Textures::GetId (T_PERLIN3));
	torus4->SetColor (1.0f, 1.0f, 1.0f, 1.0f);
	torus5 = new Torus2 (2.0f, 1.0f, 64, 64);
	torus5->SetTexture (Textures::GetId (T_PERLIN3));
	torus5->SetColor (1.0f, 1.0f, 1.0f, 1.0f);

	dlod_object = new DLODObject (torus5);
	dlod_object->AddLevel (torus4, 30.0f);
	dlod_object->AddLevel (torus3, 80.0f);
	dlod_object->AddLevel (torus2, 160.0f);
	dlod_object->AddLevel (torus1, 360.0f);
	
	// create the big torus consisting of the small toruses
	big_torus = TorusGenerator::CreateTorus (dlod_object, 100.0f, 25.0f, 32, 16);
	big_torus->CalculateRelativePositions ();
	big_torus->position->StartPath (0, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	big_torus->position->SetGlobalTransition (PP_LINEAR, 0.0f);
	big_torus->position->SetDuration (50000);
	big_torus->position->AddCheckpoint (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 75.0f);
	big_torus->position->SetDuration (10000);
	big_torus->position->SetGlobalTransition (PP_COSINE, 0.0f);
	big_torus->position->CompletePath (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1600.0f);
	AddObject (big_torus);

	// add the universe
	space = new TexturedCube (5000.0f, Textures::GetId (T_SPACE), false);
	terminal_object = new TerminalObject (space);
	terminal_object->position->SetConstantPosition (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 90.0f);
	AddObject (terminal_object);

	// set up camera
	camera = new Camera ();
	camera->SetAttributes (45, 1.333, 1.0, 8000.0);
	camera->position->StartPath (0, -7.0f, 117.0f, 3.0f, 0.0f, 10.0f, 122.0f);
	camera->position->SetDuration (9000);
	camera->position->SetGlobalTransition (PP_LINEAR, 0.0f);
	camera->position->AddCheckpoint (-7.0f, 117.0f, 3.0f, 0.0f, 10.0f, 122.0f);
	camera->position->SetDuration (5000);
	camera->position->SetGlobalTransition (PP_COSINE, 0.0f);
	camera->position->AddCheckpoint (-7.0, 117.0, 3.0f, 0.0f, 5.0f, 162.0f);
	camera->position->SetDuration (5000);
	camera->position->SetGlobalTransition (PP_SINE, 0.0f);
	camera->position->AddCheckpoint (-7.0, 117.0, 3.0f, 0.0f, 0.0f, 190.0f);
	camera->position->SetDuration (1000);
	camera->position->SetGlobalTransition (PP_LINEAR, 0.0f);
	camera->position->AddCheckpoint (-7.0, 117.0, 3.0f, 0.0f, 0.0f, 190.0f);			// Accelerate
	camera->position->SetDuration (4000);
	camera->position->SetGlobalTransition (PP_COSINE, 0.0f);
	camera->position->AddCheckpoint (-7.0f, 80.0f, 3.0f, 0.0f, 0.0f, 188.0f);			// start raising and going back
	camera->position->SetDuration (4000);
	camera->position->SetTransitionModes (PP_COSINE, PP_LINEAR, PP_COSINE, PP_LINEAR, PP_LINEAR, PP_LINEAR);
	camera->position->SetParameters (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	camera->position->AddCheckpoint (40.0f, 30.0f, 18.0f, 0.0f, 0.0f, 186.0f);			// stop, only increase x
	camera->position->SetDuration (3500);
	camera->position->SetTransitionModes (PP_LINEAR, PP_SINE, PP_SINE, PP_LINEAR, PP_LINEAR, PP_LINEAR);
	camera->position->SetParameters (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	camera->position->AddCheckpoint (85.0f, -6.0f, 30.0f, 0.0f, 0.0f, 184.0f);
	camera->position->SetDuration (2500);
	camera->position->SetGlobalTransition (PP_SINE, 0.0f);
	camera->position->AddCheckpoint (107.0f, -7.0f, 27.0f, 0.0f, 0.0f, 184.0f);
	camera->position->SetDuration (3000);
	camera->position->AddCheckpoint (107.0f, -7.0f, 27.0f, 0.0f, 0.0f, 184.0f);
	camera->position->SetDuration (7000);
	camera->position->SetTransitionModes (PP_COSINE, PP_COSINE, PP_COSINE, PP_LINEAR, PP_LINEAR, PP_LINEAR);
	camera->position->SetParameters (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	camera->position->AddCheckpoint (280.0f, -6.0f, 140.0f, 0.0f, 20.0f, 182.0f);
	camera->position->SetDuration (6000);
	camera->position->SetTransitionModes (PP_LINEAR, PP_LINEAR, PP_LINEAR, PP_SINE, PP_SINE, PP_SINE);
	camera->position->SetParameters (0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	camera->position->AddCheckpoint (600.0f, -6.0f, 280.0f, 0.0f, 27.0f, 180.0f);
	camera->position->SetDuration (10000);
	camera->position->SetGlobalTransition (PP_LINEAR, 0.0f);
	camera->position->CompletePath (600.0f, -6.0f, 280.0f, 0.0f, 27.0f, 180.0f);

	// add one light
	light = new Light ();
	light->position->StartPath (0.0f, 100.0f, 0.0f, 70.0f, 0.0f, 0.0f, 0.0f);
	light->position->SetGlobalTransition (PP_JUMP, 1.0f);
	light->position->SetDuration (60000);
	light->position->CompletePath (0.0f, 0.0f, 70.0f, 0.0f, 0.0f, 0.0f);
	light->SetAmbient (0.8f, 0.8f, 0.8f, 1.0f);
	AddLight (light);

	//PrintToConsole ();

}

void TorusScene::TorusTest (RawObject * raw_object, double r_t, double r_p, int rings, int slices)
{
	GroupObject * group;
	GroupObject * group2;
	GraphicalObject * obj;
	Point	point;

	int counter = 0;
	int max = slices / 2;
	group = new GroupObject ();

	int counter2 = 0;
	int max2 = 2;
	group2 = new GroupObject ();

	TorusGenerator::StartTorus (r_t, r_p, rings, slices);

	while (TorusGenerator::HasMore ())
	{
		obj = new TerminalObject (raw_object);
		point = TorusGenerator::GetNextPoint ();
		obj->position->SetConstantPosition (point.x, point.y, point.z, 0.0f, 0.0f, 0.0f);

		group->AddChild (obj);

		counter++;
		if (counter == max)
		{
			group2->AddChild (group);
			group = new GroupObject ();
			counter = 0;
			counter2++;
		}

		if (counter2 == max2)
		{
			AddObject (group2);
			group2 = new GroupObject ();
			counter2 = 0;
		}

	}
}